package xyz.lychee.lagfixer.libs.adventure.translation;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.function.Function;
import java.util.regex.Pattern;
import xyz.lychee.lagfixer.libs.adventure.key.Key;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

public interface TranslationRegistry extends Translator {
   Pattern SINGLE_QUOTE_PATTERN = Pattern.compile("'");

   static @NotNull TranslationRegistry create(final Key name) {
      return new TranslationRegistryImpl((Key)Objects.requireNonNull(name, "name"));
   }

   boolean contains(final @NotNull String key);

   @Nullable MessageFormat translate(final @NotNull String key, final @NotNull Locale locale);

   void defaultLocale(final @NotNull Locale locale);

   void register(final @NotNull String key, final @NotNull Locale locale, final @NotNull MessageFormat format);

   default void registerAll(final @NotNull Locale locale, final @NotNull Map<String, MessageFormat> formats) {
      Set var10002 = formats.keySet();
      Objects.requireNonNull(formats);
      this.registerAll(locale, var10002, formats::get);
   }

   default void registerAll(final @NotNull Locale locale, final @NotNull Path path, final boolean escapeSingleQuotes) {
      try {
         BufferedReader reader = Files.newBufferedReader(path, StandardCharsets.UTF_8);

         try {
            this.registerAll(locale, (ResourceBundle)(new PropertyResourceBundle(reader)), escapeSingleQuotes);
         } catch (Throwable var8) {
            if (reader != null) {
               try {
                  reader.close();
               } catch (Throwable var7) {
                  var8.addSuppressed(var7);
               }
            }

            throw var8;
         }

         if (reader != null) {
            reader.close();
         }
      } catch (IOException var9) {
      }

   }

   default void registerAll(final @NotNull Locale locale, final @NotNull ResourceBundle bundle, final boolean escapeSingleQuotes) {
      this.registerAll(locale, bundle.keySet(), (key) -> {
         String format = bundle.getString(key);
         return new MessageFormat(escapeSingleQuotes ? SINGLE_QUOTE_PATTERN.matcher(format).replaceAll("''") : format, locale);
      });
   }

   default void registerAll(final @NotNull Locale locale, final @NotNull Set<String> keys, final Function<String, MessageFormat> function) {
      IllegalArgumentException firstError = null;
      int errorCount = 0;
      Iterator var6 = keys.iterator();

      while(var6.hasNext()) {
         String key = (String)var6.next();

         try {
            this.register(key, locale, (MessageFormat)function.apply(key));
         } catch (IllegalArgumentException var9) {
            IllegalArgumentException e = var9;
            if (firstError == null) {
               firstError = e;
            }

            ++errorCount;
         }
      }

      if (firstError != null) {
         if (errorCount == 1) {
            throw firstError;
         }

         if (errorCount > 1) {
            throw new IllegalArgumentException(String.format("Invalid key (and %d more)", errorCount - 1), firstError);
         }
      }

   }

   void unregister(final @NotNull String key);
}
